(* Copyright 2010 Sarah Winkler
 * GNU Lesser General Public License
 *
 * This file is part of MKBtt.
 * 
 * MKBtt is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 * 
 * MKBtt is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with MKBtt. If not, see <http://www.gnu.org/licenses/>.
 *)

(*** SUBMODULES *********************************************************)
module SymbolSubstitution : sig
 type t
end

(*** EXCEPTIONS *********************************************************)
exception Not_renamable;;

(*** VALUES *************************************************************)
val rule_symmetric : U.Rule.t -> bool

val orientation_symmetric : 
 U.Rule.t -> U.Trs.t -> SymbolSubstitution.t

val eqns_invariant : 
 Equation.t list -> SymbolSubstitution.t -> bool

val trs_invariant : 
 U.Trs.t -> SymbolSubstitution.t -> bool

val match_termpair_sets :
  (U.Rule.t -> U.Rule.t -> 'a -> U.Rule.t * 'a list) ->
  U.Trs.t -> U.Trs.t -> 'a list -> 'a list

val match_equation_sets :
  Equation.t list -> Equation.t list -> 
  SymbolSubstitution.t list -> SymbolSubstitution.t list

val match_equations_sets :
  (Equation.t -> Equation.t -> 'a -> Equation.t * 'a list) ->
  Equation.t list -> Equation.t list -> 'a list -> 'a list

val theta_set_for_eqn_pair :
  (U.Rule.t -> U.Rule.t -> 'a -> 'b list) ->
  Equation.t -> Equation.t -> 'a -> Equation.t * 'b list

val theta_set_for_rule_pair :
  (U.Rule.t -> U.Rule.t -> 'a -> 'b) -> 
  U.Rule.t -> U.Rule.t -> 'a -> U.Rule.t * 'b

val cardinalities_match :
  Equation.t list * U.Trs.t * U.Trs.t -> 
  Equation.t list * U.Trs.t * U.Trs.t -> 
  bool

val processes_match :
  Types.CompletionProcess.t * (Equation.t list * U.Trs.t * U.Trs.t) ->
  Types.CompletionProcess.t * (Equation.t list * U.Trs.t * U.Trs.t) -> 
  bool
