(* Copyright 2008 Martin Korp, Christian Sternagel, Harald Zankl
 * GNU Lesser General Public License
 *
 * This file is part of TTT2.
 * 
 * TTT2 is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 * 
 * TTT2 is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with TTT2. If not, see <http://www.gnu.org/licenses/>.
 *)

(*** MODULES ******************************************************************)
module M = Monad;;

(*** OPENS ********************************************************************)
open Prelude;;
open Util;;

(*** MODULES ******************************************************************)
module Make (L : LABEL) = struct
 (*** MODULES *****************************************************************)
 module E = Elogic.Make (L);;
 module M = M.Make (L);;
 module Rule = Rule.Make (L);;
 module Sub = Substitution.Make (L);;
 module Term = Term.Make (L);;
 module Trs = Trs.Make (L);;

 (*** TYPES *******************************************************************)
 type 'a m = 'a M.t;;
 type term = Term.t;;
 type trs = Trs.t;;
 type t = LI | LO;;
 
 (*** FUNCTIONS ***************************************************************)
 let (>>=) = M.(>>=);;

 (* Rewrite Strategies *)
 let rewrite ?(s = LI) ?(n = ~-1) u r = (* TODO *) u;;
 
 let reducts ?(n = ~-1) u r =
  let rec reducts n us vs =
   let us' = List.rev_append vs us in
   if List.length us = List.length us' || n = 0 then us
   else
    let vs' = List.unique_hash (List.flat_map (flip Trs.reducts r) vs) in
    reducts (if n < 0 then n else n - 1) us' (List.diff vs' us')
  in
  reducts n [] [u]
 ;;
 
 let are_joinable ?(s = LI) ?(n = ~-1) u v r = (* TODO *) true;;
 
 (* Confluence *)
 let critical_pairs r =
  let cp (r,p,r') =
   let l = Rule.lhs r and l' = Rule.lhs r' and r' = Rule.rhs r in
   let s = E.unify (Term.subterm p l) l' in
   (Sub.apply_term s (Term.replace p l r'),Sub.apply_term s (Rule.rhs r))
  in
  Trs.overlaps r >>= (M.return <.> List.map cp)
 ;;
 
 let is_wcr ?(s = LI) ?(n = ~-1) r =
  critical_pairs r >>=
  (M.return <.> List.for_all (fun (u,v) -> are_joinable ~s:s ~n:n u v r))
 ;;
end
